local L = AceLibrary("AceLocale-2.2"):new("ClassLoot");

--[[ Slash command handlers ]]--
local opts = {
	type='group',
	args = {
		check = {
			type = 'text',
			name = L["Check an item (Local)"],
			desc = L["Display ClassLoot for an item locally"],
			usage = L["<item link>"],
			get = false,
			set = "ShowItemStars"
		},
		gcheck = {
			type = 'text',
			name = L["Check an item (Guild)"],
			desc = L["Display ClassLoot for an item in guild chat"],
			usage = L["<item link>"],
			get = false,
			set = "GuildCheckItem"
		},
		rcheck = {
			type = 'text',
			name = L["Check an item (Raid)"],
			desc = L["Display ClassLoot for an item in raid chat"],
			usage = L["<item link>"],
			get = false,
			set = "RaidCheckItem"
		},
		sendToRaid = {
			type = 'toggle',
			name = L["Send To Raid"],
			desc = L["Send ClassLoot info to the raid when a DKP roll starts"],
			get = function() return ClassLoot.db.profile.sendToRaid end,
			set = function(v) ClassLoot.db.profile.sendToRaid = v end
		},
		showTooltip = {
			type = 'toggle',
			name = L["Show Tooltip"],
			desc = L["Show ClassLoot Tooltip on items"],
			get = function() return ClassLoot.db.profile.showTooltip end,
			set = function(v) ClassLoot.db.profile.showTooltip = v end
		}
	}
}

--[[ Set the module up ]]--
ClassLoot = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1");
ClassLoot:RegisterChatCommand(L["Slash-Commands"], opts);

ClassLoot:RegisterDB("ClassLootDB", "ClassLootDBPerChar")
ClassLoot:RegisterDefaults("profile", {
	sendToRaid = false,
	showTooltip = true
} );

--[[ Run when ClassLoot is initialized ]]--
function ClassLoot:OnInitialize()
	-- Add the Data Updated Date to the version for /cl about
	self.version = self.version.." "..ClassLoot_Data_Updated;
end

--[[ Run when ClassLoot is enabled ]]--
function ClassLoot:OnEnable()
	-- Register events
	self:RegisterEvent("CHAT_MSG_RAID", "ChatMessage");
	self:RegisterEvent("CHAT_MSG_RAID_LEADER", "ChatMessage");
	-- Hook the tooltips
	self:HookScript(GameTooltip, "OnTooltipSetItem");
	self:HookScript(ItemRefTooltip, "OnTooltipSetItem");
end

--[[ ChatMessage Event Handler ]]--
function ClassLoot:ChatMessage(msg)
	local _, endId = string.find(msg, L["Auction-Start"]);
	if (endId and self.db.profile.sendToRaid) then
		-- We know we are in a raid so fire off the link to show the stars
		self:ShowItemStars(string.sub(msg, endId+1), 'RAID');
	end
end

function ClassLoot:GuildCheckItem(item)
	if (IsInGuild()) then
		self:ShowItemStars(item, 'GUILD');
	else
		self:Print(L["Not in a guild!"]);
	end
end

function ClassLoot:RaidCheckItem(item)
	if (GetNumRaidMembers() >= 1) then
		self:ShowItemStars(item, 'RAID');
	else
		self:Print(L["Not in a raid!"]);
	end
end

-- Get Item Info
--	link - The item link
function ClassLoot:GetItemInfoFromLink(link)
	if (type(link) == "string") then
		return ClassLoot_Data[tonumber(link:match('item:(%d+)'))];
	end
end

-- Show Item Stars
--	itemName - The link for the item
--	target   - The chat target (LOCAL, RAID or GUILD)
function ClassLoot:ShowItemStars(itemName, target)

	-- Default to LOCAL if no target is set
	local target = target or 'LOCAL';
	
	local res = AceLibrary("LibPeriodicTable-3.1"):ItemSearch(itemName);
	local BB = AceLibrary("LibBabble-Boss-3.0");
	local BZ = AceLibrary("LibBabble-Zone-3.0");
	local instance, boss = nil;
	local output = {};
	
	-- Get the info table
	local itemInfo = self:GetItemInfoFromLink(itemName);
	if (itemInfo == nil) then
		-- Not in ClassLoot_Data
		self:Print(itemName..' '..L["could not be found"]);
		return
	end
	
	-- Header
	table.insert(output, L["ClassLoot info for"]..' '..itemName);
	
	-- Who drops it where
	if (res) then
		-- k = index, v = table name [InstanceLoot.InstanceName.BossName]
		for _,v in pairs(res) do
			local set, instance, boss = strsplit(".", v);
		
			-- Three special cases where the "boss" is not the Boss!
			if (boss == "Dust Covered Chest") then boss = "Chess Event" end
			if (boss == "Cache of the Firelord") then boss = "Majordomo Executus" end
			if (boss == "Four Horsemen Chest") then boss = "The Four Horsemen" end
			
			if (set == "InstanceLoot") then
				table.insert(output, L["Dropped in"]..' '..BZ[instance]..' '..L["by"]..' '..BB[boss]);
			end
		end
	end

	-- Build starTable, index = number of stars, value = string of classes
	local starTable = { [2] = nil; [3] = nil; [4] = nil; [5] = nil; }
	for class, stars in pairs(itemInfo) do
		-- We are skipping all 0 and 1 star items as theres no point showing them
		-- After all, they shouldn't have made it into the Data file anyway!
		if (stars > 1) then
			if (starTable[stars] == nil) then
				starTable[stars] = L[class];
			else
				starTable[stars] = starTable[stars]..', '..L[class];
			end
		end
	end
	
	-- Go backwards through the table from 5 stars down to 2 and build the output
	for i=5, 2, -1 do
		if (starTable[i] ~= nil) then
			table.insert(output, self:MakeStars(i)..starTable[i]);
		end
	end
	
	-- Ouptput the Item Info!
	for _,v in pairs(output) do
		if (target == 'LOCAL') then
			self:Print(v);
		else
			SendChatMessage(v, target);
		end
	end
end

-- Make Stars
--	count - Number of stars to return
function ClassLoot:MakeStars(count)
	local stars = '';
	for i=1, 6, 1 do
		if (i <= count) then
			stars = stars..'*';
		else
			stars = stars..'  ';
		end
	end
	return stars;
end

--[[ Tooltip Hook Handler to add ClassLoot info to items ]]--
function ClassLoot:OnTooltipSetItem(tooltip, ...)
	local item = select(2, tooltip:GetItem())	
	local itemInfo = ClassLoot:GetItemInfoFromLink(item);
	if (itemInfo and self.db.profile.showTooltip) then
		-- Build starTable, index = number of stars, value = string of classes
		local starTable = { [2] = nil; [3] = nil; [4] = nil; [5] = nil; }
		for class, stars in pairs(itemInfo) do
			-- We are skipping all 0 and 1 star items as theres no point showing them
			-- After all, they shouldn't have made it into the Data file anyway!
			if (stars > 1) then
				if (starTable[stars] == nil) then
					starTable[stars] = L[class];
				else
					starTable[stars] = starTable[stars]..', '..L[class];
				end
			end
		end

		tooltip:AddLine("|cffff0000ClassLoot:|r");
		-- Go backwards through the table from 5 stars down to 2 and build the output
		for i=5, 2, -1 do
			if (starTable[i] ~= nil) then
				tooltip:AddLine("|cff0099ff"..ClassLoot:MakeStars(i)..starTable[i].."|r");
			end
		end
	end
	return self.hooks[tooltip].OnTooltipSetItem(tooltip, ...);
end